from aiohttp import web

async def translate(req):
    nmt = req['nmt']
    pipeline = req['pipeline']
    lock = req['lock']
    
    try:
        req = await req.json() # TODO sensitive point
        async with lock:
            for tu in req.get('tus'):
                src_preprocessed = pipeline.preprocess(tu['src'])
                batch_to_trans.append(src_preprocessed)

            translations = nmt.translate(batch_to_trans)

            for tu in req.get('tus'):
                tu['tgt'] = pipeline.postprocess(translations.pop(0))

        return web.json_response(req, status=200)

    except Exception as e:
        response_obj = {'status': 'failed', 'reason': str(e)}
        return web.json_response(response_obj, status=500)



from distutils.core import setup
setup(
    name = 'pangeamt_toolkit',
    packages = ['pangeamt_toolkit', 'pangeamt_toolkit.onmt',
        'pangeamt_toolkit.onmt.encoders', 'pangeamt_toolkit.onmt.decoders',
        'pangeamt_toolkit.onmt.inputters', 'pangeamt_toolkit.onmt.models',
        'pangeamt_toolkit.onmt.utils', 'pangeamt_toolkit.pangeanmt',
        'pangeamt_toolkit.pangeanmt.extended_model',
        'pangeamt_toolkit.pangeanmt.request_handler', 'pangeamt_toolkit.seg',
        'pangeamt_toolkit.processors'],
    scripts = ['bin/pangeamt_file_postprocess.py', 'bin/pangeamt_train.py',
    'bin/pangeamt_file_preprocess.py', 'bin/pangeamt_preprocess.py',
    'bin/pangeamt_translate.py'],
    version = '1.7',
    license='MIT',
    description = 'PANGEAMT-TOOLKIT',
    author = 'PANGEAMT',
    author_email = 'a.cerda@pangeanic.es',
    url = 'https://github.com/Pangeamt/pangeamt_toolkit',
    download_url = 'https://github.com/Pangeamt/pangeamt_toolkit/archive/1.7.tar.gz',
    install_requires=[
        'aiohttp',
        'async-timeout',
        'attrs',
        'certifi',
        'chardet',
        'Click',
        'ConfigArgParse',
        'future',
        'idna',
        'idna-ssl',
        'jieba',
        'joblib',
        'mecab-python3',
        'multidict',
        'numpy',
        'PyYAML',
        'requests',
        'sacremoses',
        'six',
        'subword-nmt',
        'torch',
        'torchtext',
        'tqdm',
        'typing-extensions',
        'urllib3',
        'uvloop',
        'yarl'
    ]
)



import os
import sys
import json
import uvloop
import asyncio
from aiohttp import web
from pangeamt_toolkit.pangeanmt.request_handler.save import save
from pangeamt_toolkit.pangeanmt.request_handler.train import train
from pangeamt_toolkit.pangeanmt.request_handler.translate import translate
from pangeamt_toolkit.pangeanmt import Pangeanmt

asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

current_dir = os.path.abspath(os.path.dirname(__file__))
os.chdir(current_dir)

class PangeanmtServer:
    def __init__(self, extended_model_path, new_model_path):
        app = web.Application()
        config_path = extended_model_path + '/config.json'
        with open(config_path, 'r') as file:
            config = json.loads(file.read())

        app['nmt'] = Pangeanmt(extended_model_path) # Can use config['model']
        app['new_model_path'] = new_model_path
        app['pipeline'] = Pipeline(config['pipeline_config'])
        app['lock'] = asyncio.Lock()

        app.router.add_post('/save', save)
        app.router.add_post('/train', train)
        app.router.add_post('/translate', translate)

    def start(self):
        web.run_app(app, port=8081)
